package io.sixminutes.ridicule

import android.content.Context
import com.google.android.gms.ads.identifier.AdvertisingIdClient
import com.google.android.gms.common.ConnectionResult
import com.google.android.gms.common.GoogleApiAvailability
import com.google.android.gms.common.GooglePlayServicesRepairableException
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import java.lang.Exception

class GAIDHelper(private val context: Context) {

    /**
     * 检查 Google Play 服务是否可用
     *
     * @return true 如果 Google Play 服务可用，否则返回 false
     */
    fun isGooglePlayServicesAvailable(): Boolean {
        val googleApiAvailability = GoogleApiAvailability.getInstance()
        val resultCode = googleApiAvailability.isGooglePlayServicesAvailable(context)
        return resultCode == ConnectionResult.SUCCESS
    }

    /**
     * 异步获取 GAID
     *
     * @return 获取到的 GAID
     */
    suspend fun getAdvertisingId(): String = withContext(Dispatchers.IO) {
        try {
            val adInfo = AdvertisingIdClient.getAdvertisingIdInfo(context)
            adInfo.id ?: throw NullPointerException("GAID is null")
        } catch (e: Exception) {
            throw when (e) {
                is GooglePlayServicesRepairableException ->
                    Exception("Requires Google Play services update", e)
                else -> e
            }
        }
    }

    /**
     * GAID 获取结果回调接口
     */
    interface OnAdvertisingIdListener {
        /**
         * 获取 GAID 成功回调
         *
         * @param gaid 获取到的 GAID
         */
        fun onSuccess(gaid: String)

        /**
         * 获取 GAID 失败回调
         *
         * @param e 异常信息
         */
        fun onFailure(e: Exception)
    }
}