package io.sixminutes.ridicule

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.widget.Toast
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import androidx.core.view.isVisible
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.lifecycleScope
import androidx.lifecycle.repeatOnLifecycle
import io.sixminutes.ridicule.databinding.ActivityMainBinding
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.launch

class MainActivity : AppCompatActivity() {
    private lateinit var binding: ActivityMainBinding
    private val viewModel: MainViewModel by viewModels()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)

        if (RootHelper.requestRootAccess()) {
            Toast.makeText(this, "Root 权限获取成功", Toast.LENGTH_LONG).show()
            Log.d("MainActivity", "Root access granted")
        } else {
            Toast.makeText(this, "Root 权限获取失败", Toast.LENGTH_LONG).show()
            Log.e("MainActivity", "Failed to obtain root access")
        }

        // 初始化 ViewModel
        viewModel.onViewCreated() // 调用 ViewModel 的初始化方法

        setupObservers()
        setupClickListeners()
        handleDeepLink(intent)
    }

    override fun onNewIntent(intent: Intent) {
        super.onNewIntent(intent)
        handleDeepLink(intent)
    }

    /**
     * 设置观察者
     */
    private fun setupObservers() {
        lifecycleScope.launch {
            repeatOnLifecycle(Lifecycle.State.STARTED) {
                viewModel.state.collectLatest { state ->
                    with(state) {
                        // 更新UI状态
                        binding.progressBar.isVisible = isLoading
                        binding.gaidText.text = gaid
                        binding.launchButton.isEnabled = isServiceReady
                    }
                }
            }
        }
    }

    /**
     * 设置点击监听器
     */
    private fun setupClickListeners() {
        binding.launchButton.setOnClickListener {
            if (isActive()) {
                viewModel.requestOverlayPermission(this@MainActivity)
                viewModel.launchTargetApp()
            }
        }
    }

    /**
     * 处理深度链接
     * @param intent 意图
     */
    private fun handleDeepLink(intent: Intent?) {
        intent?.data?.let { uri ->
            when (uri.path) {
                "/activate" -> viewModel.checkAccessibilityService()
            }
        }
    }

    /**
     * 检查Activity是否活跃
     * @return 是否活跃
     */
    private fun Context.isActive(): Boolean {
        return if (this is MainActivity) {
            !isFinishing && !isDestroyed && lifecycle.currentState.isAtLeast(Lifecycle.State.CREATED)
        } else {
            false
        }
    }
}
