package io.sixminutes.ridicule

import android.accessibilityservice.AccessibilityService
import android.accessibilityservice.AccessibilityServiceInfo
import android.accessibilityservice.GestureDescription
import android.annotation.SuppressLint
import android.content.Context
import android.content.Intent
import android.graphics.Path
import android.graphics.PixelFormat
import android.graphics.Rect
import android.os.Handler
import android.os.Looper
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.WindowManager
import android.view.accessibility.AccessibilityEvent
import android.view.accessibility.AccessibilityNodeInfo
import android.widget.TextView
import android.widget.Toast
import androidx.core.view.accessibility.AccessibilityNodeInfoCompat
import kotlin.random.Random

class MyAccessibilityService : AccessibilityService() {
    // region 伴生对象与实例管理
    companion object {
        private const val TAG = "AccessibilityService"
        private const val DEFAULT_MAX_RETRIES = 5
        private const val DEFAULT_RETRY_INTERVAL = 4000L

        @SuppressLint("StaticFieldLeak")
        @Volatile
        private var instance: MyAccessibilityService? = null

        /**
         * 获取服务实例（双重校验锁实现线程安全）
         * @return 当前服务实例或null（如果未初始化）
         */
        fun getInstance(): MyAccessibilityService? = instance ?: synchronized(this) {
            instance
        }
    }
    // endregion

    // region 类成员与初始化
    private var windowManager: WindowManager? = null
    private var floatingView: View? = null

    private var targetAppName: String? = null
    private var targetAppPackageName: String? = null
    private var currentAppPackageName: String? = null
    private var currentActivityName: String? = null
    private var launcherButtonText: String? = null
    private val mainHandler by lazy { Handler(Looper.getMainLooper()) }
    private var running = false

    private var isCheckAppPackageName = false  // 防止重复执行
    private var isCheckActivity = false  // 防止重复执行

    private lateinit var tapIndicatorView: TapIndicatorView

    override fun onServiceConnected() {
        super.onServiceConnected()
        tapIndicatorView = TapIndicatorView(this)

        serviceInfo.apply {
            flags = AccessibilityServiceInfo.FLAG_RETRIEVE_INTERACTIVE_WINDOWS
            eventTypes = AccessibilityEvent.TYPE_WINDOW_STATE_CHANGED
            feedbackType = AccessibilityServiceInfo.FEEDBACK_GENERIC
        }
        // 设置动态配置
        serviceInfo = serviceInfo

        instance = this
        Log.d(TAG, "Service connected")
        startMainActivitySafely()

        // 配置服务信息
        val info = AccessibilityServiceInfo().apply {
            eventTypes =
                AccessibilityEvent.TYPE_WINDOW_CONTENT_CHANGED or AccessibilityEvent.TYPE_WINDOW_STATE_CHANGED
            feedbackType = AccessibilityServiceInfo.FEEDBACK_GENERIC
            notificationTimeout = 100
        }
        serviceInfo = info
    }
    // endregion

    // region 生命周期管理
    override fun onDestroy() {
        cleanUpResources()
        super.onDestroy()

        // 移除悬浮窗口，防止内存泄漏
        removeFloatingWindow()
    }

    override fun onUnbind(intent: Intent?): Boolean {
        instance = null
        return super.onUnbind(intent)
    }

    override fun onInterrupt() {
        Log.w(TAG, "Service interrupted")
    }
    // endregion

    // region 核心功能方法
    /**
     * 模拟点击屏幕指定坐标
     * @param x X轴坐标（像素）
     * @param y Y轴坐标（像素）
     */
    private fun simulateTap(x: Int, y: Int) {
        Log.d(TAG, "Simulating tap at ($x, $y)")
        val command = "input tap $x $y"

        // **显示指示器**
        tapIndicatorView.showTapIndicator(x, y)

//        if (RootHelper.runRootCommand(command)) {
//            Toast.makeText(this, "点击 ($x, $y) 成功", Toast.LENGTH_SHORT).show()
//            Log.d(TAG, "Executed: $command")
//        } else {
//            Toast.makeText(this, "点击 ($x, $y) 失败", Toast.LENGTH_SHORT).show()
//            Log.e(TAG, "Failed to execute: $command")
//        }
    }

    /**
     * 查找并启动应用（支持后续按钮点击）
     * @param appName 应用名称（支持模糊匹配）
     * @param packageName 应用包名
     * @param buttonText 可选参数，需要点击的按钮文本
     */
    fun findAndLaunchApp(
        appName: String,
        packageName: String,
        buttonText: String? = null,
    ): Boolean {
        var success = false
        // 显示悬浮窗
        createFloatingWindow()
        safeRootOperation { root ->
            findNodesByCondition(root) { node ->
                node.textMatches(appName) || node.contentDescriptionMatches(appName)
            }.firstOrNull()?.apply {
                if (performAction(AccessibilityNodeInfo.ACTION_CLICK)) {
                    Log.d(TAG, "Successfully launched: $appName")
                    targetAppPackageName = packageName
                    launcherButtonText = buttonText
                    targetAppName = appName
                    success = true
                } else {
                    Log.w(TAG, "Failed to click app: $appName")
                    updateFloatingText("Failed to click app: $appName", true)
                }
            } ?: {
                Log.w(TAG, "App not found: $appName")
                updateFloatingText("App not found: $appName", true)
            }
        }
        return success
    }

// region 点击逻辑优化
    /**
     * 智能点击方案（综合节点属性/父容器/坐标点击）
     * @return 是否点击成功
     */
    private fun safeClickNode(node: AccessibilityNodeInfoCompat): Boolean {
        // 优先使用标准点击方式
        if (tryStandardClick(node)) return true

        // 备用方案1：尝试点击可点击的父容器
        findClickableParent(node)?.let { parent ->
            if (parent.performAction(AccessibilityNodeInfo.ACTION_CLICK)) {
                Log.d(TAG, "Clicked via parent [ID:${parent.viewIdResourceName}]")
                return true
            }
        }

        // 备用方案2：使用坐标点击
        return tryCoordinateClick(node)
    }

    /**
     * 尝试标准点击方式
     * @return 点击是否成功
     */
    private fun tryStandardClick(node: AccessibilityNodeInfoCompat): Boolean {
        return if (isNodeTrulyClickable(node)) {
            Log.d(TAG, "Attempting standard click")
            node.performAction(AccessibilityNodeInfo.ACTION_CLICK)
        } else {
            false
        }
    }

    /**
     * 增强的节点可点击性检查
     */
    private fun isNodeTrulyClickable(node: AccessibilityNodeInfoCompat): Boolean {
        // 需要同时满足多个条件（不同Android版本可能有差异）
        return node.isClickable && node.isVisibleToUser && node.isEnabled && node.isFocusable
    }

    /**
     * 深度查找可点击父容器（最多向上查找5层）
     */
    private fun findClickableParent(node: AccessibilityNodeInfoCompat): AccessibilityNodeInfoCompat? {
        var current = node.parent
        var depth = 0
        val maxDepth = 5

        while (current != null && depth < maxDepth) {
            if (isNodeTrulyClickable(current)) {
                return AccessibilityNodeInfoCompat.obtain(current)
            }
            current = current.parent
            depth++
        }
        return null
    }

    /**
     * 坐标点击（带边界校验）
     */
    private fun tryCoordinateClick(node: AccessibilityNodeInfoCompat): Boolean {
        val rect = Rect().apply {
            node.getBoundsInScreen(this)
        }

        // 边界有效性检查
        if (rect.isEmpty) {
            Log.w(TAG, "Invalid node bounds")
            return false
        }

        val screenMetrics = getScreenMetrics()
        if (!rect.intersect(screenMetrics)) {
            Log.w(TAG, "Node out of screen bounds")
            return false
        }

        return dispatchGesture(
            createClickGesture(rect.centerX(), rect.centerY()), null, null
        )
    }

    /**
     * 创建点击手势描述
     */
    private fun createClickGesture(x: Int, y: Int): GestureDescription {
        val path = Path().apply { moveTo(x.toFloat(), y.toFloat()) }
        return GestureDescription.Builder().addStroke(
            GestureDescription.StrokeDescription(
                path, 0L,      // 开始时间
                50L      // 持续时间（毫秒）
            )
        ).build()
    }

    /**
     * 获取屏幕实际显示区域
     */
    private fun getScreenMetrics(): Rect {
        val metrics = resources.displayMetrics
        return Rect(0, 0, metrics.widthPixels, metrics.heightPixels)
    }
// endregion

    // region 节点操作工具方法
    private fun findAndClickButton(
        buttonText: String,
        maxRetries: Int = DEFAULT_MAX_RETRIES,
        retryInterval: Long = DEFAULT_RETRY_INTERVAL
    ): Boolean {
        Log.d(TAG, "Attempting to click button: '$buttonText'")
        updateFloatingText("Attempting to click button: '$buttonText'")
        var success = false

        safeRootOperation { root ->
            retryOperation(maxRetries = maxRetries, interval = retryInterval, checkCondition = {
                // 精确查找可点击节点（包含回收保障）
                var found = false
                findNodesByCondition(root) { node ->
                    node.textMatches(
                        buttonText, exact = true
                    ) || node.contentDescriptionMatches(buttonText, exact = true)
                }.firstOrNull()?.apply {
                    Log.d(
                        TAG, "Found clickable button: '$buttonText' [ID:${viewIdResourceName}]"
                    )
                    found = safeClickNode(this)
                }
                Log.d(TAG, "$buttonText is clicked: $found")
                found
            }, onSuccess = {
                success = true
                Log.i(TAG, "Successfully clicked button: '$buttonText'")
            }, onFailure = {
                Log.w(TAG, "Failed to click button after $maxRetries attempts: '$buttonText'")
                showToast("Failed to click $buttonText")
            })
        }

        return success
    }

    /**
     * 通用重试操作模板
     * @param maxRetries 最大重试次数
     * @param interval 重试间隔
     * @param checkCondition 需要检查的条件（返回Boolean表示是否成功）
     * @param onSuccess 成功回调
     * @param onFailure 失败回调
     */
    private fun retryOperation(
        maxRetries: Int,
        interval: Long,
        checkCondition: () -> Boolean,
        onSuccess: () -> Unit,
        onFailure: () -> Unit
    ) {
        var retryCount = 0

        fun attempt() {
            if (!running) return
            when {
                checkCondition() -> onSuccess()
                retryCount < maxRetries -> {
                    retryCount++
                    Log.d(TAG, "Retry attempt $retryCount")
                    mainHandler.postDelayed(::attempt, interval)
                }

                else -> onFailure()
            }
        }

        attempt()
    }

    /**
     * 安全执行根节点操作（自动处理资源回收）
     * @param block 需要在根节点上下文中执行的操作
     */

    private inline fun safeRootOperation(block: (AccessibilityNodeInfoCompat) -> Unit) {
        try {
            rootInActiveWindow?.let { rawRoot ->
                val wrappedRoot = AccessibilityNodeInfoCompat.wrap(rawRoot)

                block(wrappedRoot)

            } ?: Log.e(TAG, "No active window available")
        } catch (e: Exception) {
            Log.e(TAG, "Root operation failed", e)
        }
    }

    // region 节点操作工具方法
    /**
     * 递归查找符合条件的节点
     * @param root 起始节点
     * @param predicate 节点匹配条件
     * @return 匹配的节点列表（需要手动回收）
     */
    private fun findNodesByCondition(
        root: AccessibilityNodeInfoCompat, predicate: (AccessibilityNodeInfoCompat) -> Boolean
    ): List<AccessibilityNodeInfoCompat> {
        val result = mutableListOf<AccessibilityNodeInfoCompat>()

        fun traverse(node: AccessibilityNodeInfoCompat) {
            if (predicate(node)) result.add(AccessibilityNodeInfoCompat.obtain(node))
            (0 until node.childCount).mapNotNull { node.getChild(it) }.forEach {
                traverse(it)
            }
        }

        traverse(root)
        return result
    }

    /**
     * 文本匹配扩展方法
     * @param text 要匹配的文本
     * @param exact 是否精确匹配（默认false）
     */
    private fun AccessibilityNodeInfoCompat.textMatches(
        text: String, exact: Boolean = false
    ): Boolean {
        return this.text?.toString()?.let {
            if (exact) it.equals(text, true) else it.contains(text, true)
        } ?: false
    }

    /**
     * 内容描述匹配扩展方法
     * @param text 要匹配的文本
     * @param exact 是否精确匹配（默认false）
     */
    private fun AccessibilityNodeInfoCompat.contentDescriptionMatches(
        text: String, exact: Boolean = false
    ): Boolean {
        return this.contentDescription?.toString()?.let {
            if (exact) it.equals(text, true) else it.contains(text, true)
        } ?: false
    }
    // endregion

    // region 其他辅助方法
    private fun showToast(message: String) {
        mainHandler.post { Toast.makeText(this, message, Toast.LENGTH_LONG).show() }
    }

    private fun startMainActivitySafely() {
        try {
            startActivity(Intent(this, MainActivity::class.java).apply {
                flags = Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK
            })
        } catch (e: Exception) {
            Log.e(TAG, "Start MainActivity failed", e)
        }
    }

    private fun cleanUpResources() {
        instance = null
        targetAppPackageName = null
    }
    // endregion

    // region 无障碍事件处理
    override fun onAccessibilityEvent(event: AccessibilityEvent?) {
        event?.let {
            if (event.eventType == AccessibilityEvent.TYPE_WINDOW_STATE_CHANGED) handleWindowChange(
                event
            )
            // 可根据需要扩展其他事件类型处理
        }
    }

    // 检查当前窗口是哪个
    private fun isTargetAppRunning(): Boolean {
        val currentRoot = rootInActiveWindow
        return if (currentRoot != null) {
            currentAppPackageName = currentRoot.packageName?.toString()
            currentAppPackageName == targetAppPackageName

        } else {
            Log.e(TAG, "isTargetAppRunning: rootInActiveWindow is null")
            updateFloatingText("rootInActiveWindow is null", true)
            false
        }
    }

    private fun handleWindowChange(event: AccessibilityEvent) {
        if (!running) return
        if (targetAppPackageName == null) return

        val packageName = event.packageName?.toString() ?: "unknownPackageName"
        val activity = event.className?.toString() ?: "unknownActivity"

        if (packageName == "io.sixminutes.ridicule" && activity == "android.widget.LinearLayout") {
            Log.w(TAG, "Detected current app is floating window")
            return
        }

        // 增强日志输出
        Log.d(
            TAG, """
        Window Changed:
        ├─ Current Package : $packageName
        ├─ Target Package  : $targetAppPackageName
        ├─ Class Name      : ${activity.substringAfterLast('.')}
        └─ Full Class Name : $activity
    """.trimIndent()
        )

        currentAppPackageName = packageName
        // 当检测到切换到其他应用时
        if (currentAppPackageName != targetAppPackageName) {
            checkAppPackageName()
        } else {
            currentActivityName = activity
            checkActivity()
        }
    }

    // 检查前端应用
    private fun checkAppPackageName() {
        if (isCheckAppPackageName) {
            Log.w(TAG, "CheckAppPackageName operation is already running, skipping...")
            updateFloatingText("CheckAppPackageName operation is already running, skipping...")
            return
        }

        isCheckAppPackageName = true
        Log.w(TAG, "Detected switch to non-target app: $currentAppPackageName")
        updateFloatingText("Detected switch to non-target app: $currentAppPackageName")

        retryOperation(DEFAULT_MAX_RETRIES, DEFAULT_RETRY_INTERVAL, checkCondition = {
            val ok = isTargetAppRunning()
            ok || performReturnGlobalAction()
            Log.d(TAG, "Check $targetAppPackageName is activated: $ok")
            ok
        }, onSuccess = {
            isCheckAppPackageName = false  // 任务结束后重置
            updateFloatingText("Successfully returned to target app")
            Log.i(TAG, "Successfully returned to target app")
            // 返回后再次检查 Activity
            mainHandler.postDelayed(::checkActivity, DEFAULT_RETRY_INTERVAL)
        }, onFailure = {
            isCheckAppPackageName = false  // 任务结束后重置
            updateFloatingText("Failed to return after $DEFAULT_MAX_RETRIES attempts", true)
            Log.e(TAG, "Failed to return after $DEFAULT_MAX_RETRIES attempts")
            returnToHomeAndRestart()
        })
    }

    // 检查当前 Activity
    private fun checkActivity() {
        if (isCheckActivity) {
            Log.w(TAG, "CheckActivity operation is already running, skipping...")
            updateFloatingText("CheckActivity operation is already running, skipping...")
            return
        }
        isCheckActivity = true
        Log.w(TAG, "Current class name is : $currentActivityName")
        try {
            when (currentActivityName) {
                "io.sixminutes.breakingnews.MainActivity" -> launcherButtonText?.let {
                    findAndClickButton(it)
                }

                "io.sixminutes.breakingnews.InterstitialActivity" -> Log.d(
                    TAG, "current Activity is InterstitialActivity"
                )

                "io.sixminutes.breakingnews.ClickTrackerActivity" -> simulateTap(
                    Random.nextInt(
                        0, 720
                    ), Random.nextInt(0, 1080)
                )

                "com.applovin.adview.AppLovinFullscreenActivity" -> simulateTap(648, 62)
                else -> {
                    Log.d(TAG, "current Activity is unknown")
                    updateFloatingText("current Activity is unknown")
                    mainHandler.postDelayed({
                        checkActivity()
                    }, DEFAULT_RETRY_INTERVAL)
                }
            }
        } finally {
            isCheckActivity = false
        }
    }

    // endregion

    // region 返回策略实现
    /**
     * 按返回键
     */
    private fun performReturnGlobalAction(): Boolean {
        if (performGlobalAction(GLOBAL_ACTION_BACK)) {
            Log.d(TAG, "Attempted back navigation")
            return true
        }

        return false
    }

    /**
     * 返回桌面并重启应用
     */
    private fun returnToHomeAndRestart(): Boolean {
        Log.d(TAG, "Used home+restart strategy")
        updateFloatingText("used home+restart strategy")
        return performGlobalAction(GLOBAL_ACTION_HOME) && targetAppName != null && targetAppPackageName != null && findAndLaunchApp(
            targetAppName!!, targetAppPackageName!!, launcherButtonText
        )
    }

    // endregion

    // region 县浮窗
    /**
     * 创建悬浮窗口，并添加到 WindowManager
     */
    @SuppressLint("InflateParams")
    private fun createFloatingWindow() {
        if (running) return
        running = true

        // 获取 WindowManager 服务
        windowManager = getSystemService(Context.WINDOW_SERVICE) as WindowManager
        if (floatingView == null) {
            // 使用 LayoutInflater 加载悬浮窗口布局
            floatingView = LayoutInflater.from(this).inflate(R.layout.floating_window, null, false)
            // 设置点击监听器，点击时移除悬浮窗口
            floatingView?.setOnClickListener {
                hideFloatingWindow()
            }
        }
        // 设置窗口布局参数
        val layoutParams = WindowManager.LayoutParams(
            WindowManager.LayoutParams.WRAP_CONTENT,
            WindowManager.LayoutParams.WRAP_CONTENT,
            WindowManager.LayoutParams.TYPE_APPLICATION_OVERLAY,
            // FLAG_NOT_FOCUSABLE 表示窗口不获取焦点，FLAG_LAYOUT_IN_SCREEN 表示全屏显示
            WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE or WindowManager.LayoutParams.FLAG_LAYOUT_IN_SCREEN or WindowManager.LayoutParams.FLAG_LAYOUT_NO_LIMITS,
            PixelFormat.TRANSLUCENT
        )
        // 设置悬浮窗口显示在屏幕的左上角，x,y 为偏移量
        layoutParams.gravity = android.view.Gravity.BOTTOM or android.view.Gravity.END
        layoutParams.x = 0
        layoutParams.y = 150

        // 将悬浮窗口添加到 WindowManager 中显示
        windowManager?.addView(floatingView, layoutParams)
    }

    /**
     * 隐藏悬浮窗并回到主窗口
     */
    private fun hideFloatingWindow() {
        if (!running) return
        running = false

        try {
            floatingView?.let {
                // 获取 WindowManager 服务
                windowManager?.removeView(it)
            }

            // 启动 MainActivity
            val intent = Intent(this, MainActivity::class.java).apply {
                flags =
                    Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TOP or Intent.FLAG_ACTIVITY_REORDER_TO_FRONT or Intent.FLAG_ACTIVITY_BROUGHT_TO_FRONT
            }
            startActivity(intent)
        } catch (e: Exception) {
            Log.e("MainActivity", "Hide floating window failed", e)
        }
    }

    /**
     * 更新悬浮窗口中显示的文本
     * @param text 要显示的文本内容
     */
    private fun updateFloatingText(text: String, isError: Boolean = false) {
        // 在悬浮窗口中找到 TextView 并设置新文本
        if (isError) {
            val errorView = floatingView?.findViewById<TextView>(R.id.error_message)
            errorView?.text = text
        } else {
            val info1 = floatingView?.findViewById<TextView>(R.id.info_message_1)
            val info2 = floatingView?.findViewById<TextView>(R.id.info_message_2)
            info2?.text = info1?.text
            info1?.text = text
        }
    }

    /**
     * 移除悬浮窗口，释放资源
     */
    private fun removeFloatingWindow() {
        if (floatingView != null) {
            windowManager?.removeView(floatingView)
            floatingView = null
        }
    }
    //endregion
}
