package io.sixminutes.ridicule.monitor

import android.content.Context
import de.robv.android.xposed.IXposedHookLoadPackage
import de.robv.android.xposed.XC_MethodHook
import de.robv.android.xposed.XposedBridge
import de.robv.android.xposed.XposedHelpers
import de.robv.android.xposed.callbacks.XC_LoadPackage

class AdIdMonitor : IXposedHookLoadPackage {

    companion object {
        private const val TARGET_PACKAGE = "io.sixminutes.breakingnews"
        private const val LOG_TAG = "AdIdTracker"
        private const val STACK_DEPTH = 10 // 堆栈跟踪深度
    }

    override fun handleLoadPackage(lpparam: XC_LoadPackage.LoadPackageParam) {
        if (lpparam.packageName != TARGET_PACKAGE) return

        hookAdvertisingIdClient(lpparam.classLoader)
        hookReflectiveCalls(lpparam.classLoader)
    }

    // region 核心 Hook 逻辑
    private fun hookAdvertisingIdClient(classLoader: ClassLoader) {
        // Hook AdvertisingIdClient.getAdvertisingIdInfo()
        val adIdClientClass = "com.google.android.gms.ads.identifier.AdvertisingIdClient"
        XposedHelpers.findAndHookMethod(
            adIdClientClass,
            classLoader,
            "getAdvertisingIdInfo",
            Context::class.java,
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    val info = param.result
                    logCall(
                        method = "AdvertisingIdClient.getAdvertisingIdInfo",
                        result = info?.javaClass?.name,
                        stackDepth = STACK_DEPTH
                    )

                    // Hook Info.getId()
                    XposedHelpers.findAndHookMethod(
                        info?.javaClass,
                        "getId",
                        object : XC_MethodHook() {
                            override fun afterHookedMethod(param: MethodHookParam) {
                                logCall(
                                    method = "AdvertisingIdInfo.getId",
                                    result = param.result,
                                    stackDepth = 5
                                )
                            }
                        }
                    )
                }
            }
        )
    }

    private fun hookReflectiveCalls(classLoader: ClassLoader) {
        // Hook Class.forName() 防止反射调用绕过
        XposedHelpers.findAndHookMethod(
            Class::class.java,
            "forName",
            String::class.java,
            Boolean::class.java,
            ClassLoader::class.java,
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    val className = param.args[0] as String
                    if (className.contains("AdvertisingIdClient")) {
                        logCall(
                            method = "Reflective: Class.forName($className)",
                            result = param.result,
                            stackDepth = STACK_DEPTH
                        )
                    }
                }
            }
        )
    }
    // endregion

    // region 日志工具
    private fun logCall(method: String, result: Any?, stackDepth: Int = 0) {
        val stackTrace = if (stackDepth > 0) {
            "\nStackTrace:\n${
                Thread.currentThread().stackTrace
                    .drop(4) // 跳过 Xposed 内部调用
                    .take(stackDepth)
                    .joinToString("\n") { "  at ${it.className}.${it.methodName} (${it.fileName}:${it.lineNumber})" }
            }"
        } else ""

        val logMsg = """
        [][$LOG_TAG] $method
        → Result: $result
        $stackTrace
        """.trimIndent()

        XposedBridge.log(logMsg)
    }
    // endregion
}

//   Loading class io.sixminutes.ridicule.AdIdTracker
//         [][AdIdTracker] Reflective: Class.forName(com.google.android.gms.ads.identifier.AdvertisingIdClient)
//         → Result: class com.google.android.gms.ads.identifier.AdvertisingIdClient

// StackTrace:
//   at io.sixminutes.ridicule.AdIdTracker$hookReflectiveCalls$1.afterHookedMethod (AdIdTracker.kt:74)
//   at vTxl.pKyTPAWbQR.SNM.in.XposedBridge$LegacyApiSupport.handleAfter (null:33)
//   at J.callback (null:292)
//   at LSPHooker_.forName (null:18)
//   at java.lang.Class.forName (Class.java:379)
//   at com.applovin.impl.d7.a (SourceFile:277)
//   at com.applovin.impl.r.a (SourceFile:86)
//   at com.applovin.impl.r.collectGoogleAdvertisingInfo (SourceFile:3)
//   at com.applovin.impl.r.c (SourceFile:1)
//   at com.applovin.impl.r.a (SourceFile:30)
//         [][AdIdTracker] Reflective: Class.forName(com.google.android.gms.ads.identifier.AdvertisingIdClient)
//         → Result: class com.google.android.gms.ads.identifier.AdvertisingIdClient

// StackTrace:
//   at io.sixminutes.ridicule.AdIdTracker$hookReflectiveCalls$1.afterHookedMethod (AdIdTracker.kt:74)
//   at vTxl.pKyTPAWbQR.SNM.in.XposedBridge$LegacyApiSupport.handleAfter (null:33)
//   at J.callback (null:292)
//   at LSPHooker_.forName (null:18)
//   at java.lang.Class.forName (Class.java:379)
//   at com.applovin.impl.d7.a (SourceFile:277)
//   at com.applovin.impl.r.a (SourceFile:86)
//   at com.applovin.impl.sdk.k.z (SourceFile:7)
//   at com.applovin.impl.sdk.k.<init> (SourceFile:33)
//   at com.applovin.impl.sdk.j.x (SourceFile:1)
//         [][AdIdTracker] AdvertisingIdClient.getAdvertisingIdInfo
//         → Result: com.google.android.gms.ads.identifier.AdvertisingIdClient$Info

// StackTrace:
//   at io.sixminutes.ridicule.AdIdTracker$hookAdvertisingIdClient$1.afterHookedMethod (AdIdTracker.kt:37)
//   at vTxl.pKyTPAWbQR.SNM.in.XposedBridge$LegacyApiSupport.handleAfter (null:33)
//   at J.callback (null:292)
//   at LSPHooker_.getAdvertisingIdInfo (null:8)
//   at com.applovin.impl.r.collectGoogleAdvertisingInfo (SourceFile:9)
//   at com.applovin.impl.r.c (SourceFile:1)
//   at com.applovin.impl.r.a (SourceFile:30)
//   at com.applovin.impl.r.b (SourceFile:1)
//   at com.applovin.impl.d1.b (SourceFile:1)
//   at com.applovin.impl.d1.$r8$lambda$l4cN_DUyjuxgoDSaHdcYRuuV-kI (null:0)
//         [][AdIdTracker] AdvertisingIdInfo.getId
//         → Result: 82f43678-ac7f-4eeb-b518-7accb31529f6

// StackTrace:
//   at io.sixminutes.ridicule.AdIdTracker$hookAdvertisingIdClient$1$afterHookedMethod$1.afterHookedMethod (AdIdTracker.kt:49)
//   at vTxl.pKyTPAWbQR.SNM.in.XposedBridge$LegacyApiSupport.handleAfter (null:33)
//   at J.callback (null:292)
//   at LSPHooker_.getId (null:8)
//   at com.applovin.impl.r.collectGoogleAdvertisingInfo (SourceFile:13)
