package io.sixminutes.ridicule.monitor

import android.content.pm.PackageManager
import de.robv.android.xposed.IXposedHookLoadPackage
import de.robv.android.xposed.XC_MethodHook
import de.robv.android.xposed.XposedBridge
import de.robv.android.xposed.XposedHelpers
import de.robv.android.xposed.callbacks.XC_LoadPackage

class CarrierMonitor2 : IXposedHookLoadPackage {

    companion object {
        private const val TARGET_PACKAGE = "io.sixminutes.breakingnews"
        private const val LOG_TAG = "CarrierTracker"
        private const val STACK_DEPTH = 8 // 堆栈跟踪深度
    }

    override fun handleLoadPackage(lpparam: XC_LoadPackage.LoadPackageParam) {
        if (lpparam.packageName != TARGET_PACKAGE) return

        hookTelephonyApis(lpparam.classLoader)
        hookSubscriptionApis(lpparam.classLoader)
        hookLocaleApis(lpparam.classLoader)
        bypassPermissionChecks(lpparam.classLoader)
    }

    // region 核心 Hook 逻辑
    private fun hookTelephonyApis(classLoader: ClassLoader) {
        val tmClass = "android.telephony.TelephonyManager"

        // Hook 运营商信息
        arrayOf(
            "getNetworkOperator",      // MCC+MNC
            "getSimOperator",          // SIM MCC+MNC
            "getNetworkOperatorName",  // 运营商名称
            "getNetworkCountryIso",    // 网络国家代码
            "getSimCountryIso"         // SIM 国家代码
        ).forEach { method ->
            XposedHelpers.findAndHookMethod(
                tmClass,
                classLoader,
                method,
                object : XC_MethodHook() {
                    override fun afterHookedMethod(param: MethodHookParam) {
                        logCall(
                            caller = tmClass,
                            method = method,
                            result = param.result,
                            stackDepth = STACK_DEPTH
                        )
                    }
                }
            )
        }

        // Hook ServiceState
        XposedHelpers.findAndHookMethod(
            "android.telephony.ServiceState",
            classLoader,
            "getOperatorNumeric",
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    logCall(
                        caller = "ServiceState",
                        method = "getOperatorNumeric",
                        result = param.result
                    )
                }
            }
        )
    }

    private fun hookSubscriptionApis(classLoader: ClassLoader) {
        // Hook SubscriptionManager (多卡)
        XposedHelpers.findAndHookMethod(
            "android.telephony.SubscriptionManager",
            classLoader,
            "getActiveSubscriptionInfoList",
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    (param.result as? List<*>)?.forEach { subInfo ->
                        val mcc = XposedHelpers.callMethod(subInfo, "getMcc")
                        val mnc = XposedHelpers.callMethod(subInfo, "getMnc")
                        val countryIso = XposedHelpers.callMethod(subInfo, "getCountryIso")
                        logCall(
                            caller = "SubscriptionInfo",
                            method = "getMcc/getMnc/getCountryIso",
                            result = "MCC=$mcc, MNC=$mnc, Country=$countryIso"
                        )
                    }
                }
            }
        )
    }

    private fun hookLocaleApis(classLoader: ClassLoader) {
        // Hook 区域设置获取国家
        XposedHelpers.findAndHookMethod(
            "java.util.Locale",
            classLoader,
            "getCountry",
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    logCall(
                        caller = "Locale",
                        method = "getCountry",
                        result = param.result,
                        stackDepth = STACK_DEPTH
                    )
                }
            }
        )
    }

    private fun bypassPermissionChecks(classLoader: ClassLoader) {
        // 绕过 READ_PHONE_STATE 权限
        XposedHelpers.findAndHookMethod(
            "android.app.ContextImpl",
            classLoader,
            "checkPermission",
            String::class.java,
            Int::class.java,
            Int::class.java,
            object : XC_MethodHook() {
                override fun beforeHookedMethod(param: MethodHookParam) {
                    if (param.args[0] == android.Manifest.permission.READ_PHONE_STATE) {
                        param.result = PackageManager.PERMISSION_GRANTED
                    }
                }
            }
        )
    }
    // endregion

    // region 日志工具
    private fun logCall(caller: String, method: String, result: Any?, stackDepth: Int = 0) {
        val stackTrace = if (stackDepth > 0) {
            "\nStackTrace:\n${
                Thread.currentThread().stackTrace
                    .drop(4) // 跳过 Xposed 内部调用
                    .take(stackDepth)
                    .joinToString("\n") { "  at ${it.className}.${it.methodName} (${it.fileName}:${it.lineNumber})" }
            }"
        } else ""

        val logMsg = """
        [][$LOG_TAG] $caller.$method
        → Result: $result
        $stackTrace
        """.trimIndent()

        XposedBridge.log(logMsg)
    }
    // endregion
}

//    Loading class io.sixminutes.ridicule.monitor.CarrierMonitor2
//          [][CarrierTracker] Locale.getCountry
//          → Result: CN

//  StackTrace:
//    at io.sixminutes.ridicule.monitor.CarrierMonitor2$hookLocaleApis$1.afterHookedMethod (CarrierMonitor2.kt:104)
//    at vTxl.pKyTPAWbQR.SNM.in.XposedBridge$LegacyApiSupport.handleAfter (null:33)
//    at J.callback (null:292)
//    at LSPHooker_.getCountry (null:8)
//    at java.text.DecimalFormatSymbols.initialize (DecimalFormatSymbols.java:701)
//    at java.text.DecimalFormatSymbols.<init> (DecimalFormatSymbols.java:102)
//    at java.text.DecimalFormatSymbols.getInstance (DecimalFormatSymbols.java:156)
//    at java.text.DecimalFormat.<init> (DecimalFormat.java:422)
//          [][CarrierTracker] Locale.getCountry
//          → Result: CN

//  StackTrace:
//    at io.sixminutes.ridicule.monitor.CarrierMonitor2$hookLocaleApis$1.afterHookedMethod (CarrierMonitor2.kt:104)
//    at vTxl.pKyTPAWbQR.SNM.in.XposedBridge$LegacyApiSupport.handleAfter (null:33)
//    at J.callback (null:292)
//    at LSPHooker_.getCountry (null:8)
//    at java.util.Currency.getInstance (Currency.java:321)
//    at java.text.DecimalFormatSymbols.initialize (DecimalFormatSymbols.java:703)
//    at java.text.DecimalFormatSymbols.<init> (DecimalFormatSymbols.java:102)
//    at java.text.DecimalFormatSymbols.getInstance (DecimalFormatSymbols.java:156)
//          [][CarrierTracker] android.telephony.TelephonyManager.getSimCountryIso
//          → Result: us

//  StackTrace:
//    at io.sixminutes.ridicule.monitor.CarrierMonitor2$hookTelephonyApis$1$1.afterHookedMethod (CarrierMonitor2.kt:45)
//    at vTxl.pKyTPAWbQR.SNM.in.XposedBridge$LegacyApiSupport.handleAfter (null:33)
//    at J.callback (null:292)
//    at LSPHooker_.getSimCountryIso (null:8)
//    at com.applovin.impl.sdk.k.k (SourceFile:2)
//    at com.applovin.impl.sdk.k.z (SourceFile:15)
//    at com.applovin.impl.sdk.k.<init> (SourceFile:33)
//    at com.applovin.impl.sdk.j.x (SourceFile:1)
//          [][CarrierTracker] android.telephony.TelephonyManager.getNetworkOperator
//          → Result: 310260

//  StackTrace:
//    at io.sixminutes.ridicule.monitor.CarrierMonitor2$hookTelephonyApis$1$1.afterHookedMethod (CarrierMonitor2.kt:45)
//    at vTxl.pKyTPAWbQR.SNM.in.XposedBridge$LegacyApiSupport.handleAfter (null:33)
//    at J.callback (null:292)
//    at LSPHooker_.getNetworkOperator (null:8)
//    at com.applovin.impl.sdk.k.u (SourceFile:6)
//    at com.applovin.impl.sdk.k.z (SourceFile:16)
//    at com.applovin.impl.sdk.k.<init> (SourceFile:33)
//    at com.applovin.impl.sdk.j.x (SourceFile:1)
//          [][CarrierTracker] android.telephony.TelephonyManager.getNetworkOperator
//          → Result: 310260

//  StackTrace:
//    at io.sixminutes.ridicule.monitor.CarrierMonitor2$hookTelephonyApis$1$1.afterHookedMethod (CarrierMonitor2.kt:45)
//    at vTxl.pKyTPAWbQR.SNM.in.XposedBridge$LegacyApiSupport.handleAfter (null:33)
//    at J.callback (null:292)
//    at LSPHooker_.getNetworkOperator (null:8)
//    at com.applovin.impl.sdk.k.v (SourceFile:6)
//    at com.applovin.impl.sdk.k.z (SourceFile:17)
//    at com.applovin.impl.sdk.k.<init> (SourceFile:33)
//    at com.applovin.impl.sdk.j.x (SourceFile:1)
//          [][CarrierTracker] android.telephony.TelephonyManager.getNetworkOperatorName
//          → Result: T-Mobile

//  StackTrace:
//    at io.sixminutes.ridicule.monitor.CarrierMonitor2$hookTelephonyApis$1$1.afterHookedMethod (CarrierMonitor2.kt:45)
//    at vTxl.pKyTPAWbQR.SNM.in.XposedBridge$LegacyApiSupport.handleAfter (null:33)
//    at J.callback (null:292)
//    at LSPHooker_.getNetworkOperatorName (null:8)
//    at com.applovin.impl.sdk.k.i (SourceFile:8)
//    at com.applovin.impl.sdk.k.z (SourceFile:18)
//    at com.applovin.impl.sdk.k.<init> (SourceFile:33)
//    at com.applovin.impl.sdk.j.x (SourceFile:1)
//          [][CarrierTracker] Locale.getCountry
//          → Result: CN

//  StackTrace:
//    at io.sixminutes.ridicule.monitor.CarrierMonitor2$hookLocaleApis$1.afterHookedMethod (CarrierMonitor2.kt:104)
//    at vTxl.pKyTPAWbQR.SNM.in.XposedBridge$LegacyApiSupport.handleAfter (null:33)
//    at J.callback (null:292)
//    at LSPHooker_.getCountry (null:8)
//    at android.content.res.Configuration.compareTo (Configuration.java:2074)
//    at android.content.res.Configuration.equals (Configuration.java:2127)
//    at android.content.res.Configuration.equals (Configuration.java:2132)
//    at java.util.Objects.equals (Objects.java:76)
//          [][CarrierTracker] Locale.getCountry
//          → Result: CN
