package io.sixminutes.ridicule.monitor

import de.robv.android.xposed.IXposedHookLoadPackage
import de.robv.android.xposed.XC_MethodHook
import de.robv.android.xposed.XposedBridge
import de.robv.android.xposed.XposedHelpers
import de.robv.android.xposed.callbacks.XC_LoadPackage

class MethodMonitor : IXposedHookLoadPackage {

    // 配置参数（按需修改）
    companion object {
        private const val TARGET_PACKAGE = "io.sixminutes.breakingnews"
        private const val LOG_TAG = "MethodTracer"
        private const val ENABLE_STACK_TRACE = true // 是否记录调用堆栈
        private val EXCLUDED_CLASSES = listOf(      // 排除的类
            "java.", "kotlin."
        )
    }

    override fun handleLoadPackage(lpparam: XC_LoadPackage.LoadPackageParam) {
//        if (lpparam.packageName != TARGET_PACKAGE) return

        // 监听类加载
        XposedHelpers.findAndHookMethod(
            ClassLoader::class.java,
            "loadClass",
            String::class.java,
            Boolean::class.java,
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    val className = param.args[0] as String
                    val clazz = param.result as? Class<*> ?: return

                    // 过滤排除的类
                    if (EXCLUDED_CLASSES.any { className.startsWith(it) }) return

                    // Hook 类中的所有方法
                    hookAllMethods(clazz, lpparam.classLoader)
                }
            }
        )
    }

    /**
     * Hook 指定类的所有方法
     */
    private fun hookAllMethods(clazz: Class<*>, classLoader: ClassLoader) {
        try {
            for (method in clazz.declaredMethods) {
                XposedBridge.hookMethod(method, object : XC_MethodHook() {
                    override fun beforeHookedMethod(param: MethodHookParam) {
                        logMethodCall(
                            clazz = clazz.name,
                            method = method.name,
                            args = param.args,
                            isEntry = true
                        )
                    }

                    override fun afterHookedMethod(param: MethodHookParam) {
                        logMethodCall(
                            clazz = clazz.name,
                            method = method.name,
                            args = param.args,
                            isEntry = false,
                            result = param.result
                        )
                    }
                })
            }
        } catch (e: Throwable) {
            XposedBridge.log("[$LOG_TAG] Error hooking ${clazz.name}: ${e.javaClass.simpleName}")
        }
    }

    /**
     * 记录方法调用日志
     */
    private fun logMethodCall(
        clazz: String,
        method: String,
        args: Array<Any>?,
        isEntry: Boolean,
        result: Any? = null
    ) {
        val logBuilder = StringBuilder().apply {
            append(if (isEntry) "→ ENTER" else "← EXIT")
            append(" $clazz.$method")

            // 记录参数
            args?.let {
                append("(")
                it.forEachIndexed { i, arg ->
                    append("arg$i=${arg?.javaClass?.simpleName ?: "null"}")
                    if (i < it.size - 1) append(", ")
                }
                append(")")
            }

            // 记录返回值
            if (!isEntry) {
                append(" → ")
                append(result?.javaClass?.simpleName ?: "void")
            }
        }

        // 输出到 Xposed 日志
        XposedBridge.log("[$LOG_TAG] ${logBuilder}")

        // 记录调用堆栈（可选）
        if (ENABLE_STACK_TRACE && isEntry) {
            val stackTrace = Thread.currentThread().stackTrace
                .drop(3) // 忽略 Xposed 内部调用
                .take(5) // 只取前5层
                .joinToString("\n") { "    at ${it.className}.${it.methodName} (${it.fileName}:${it.lineNumber})" }
            XposedBridge.log("[$LOG_TAG] StackTrace:\n$stackTrace")
        }
    }
}